// *********
// Settings
// *********
class SettingsClass {
    constructor() {

    }
}
var Settings = new SettingsClass();

function SettingsToJSON() {
    return JSON.stringify(Settings);
}

function JSONToSettings(json) {
    Settings = JSON.parse(json);
}

// *********
// Vec2
// *********
class Vec2 {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    normalize() {
        var n = this.length();
        if (n == 0) {
            return new Vec2(0, 0);
        }
        n = 1.0 / n;
        return new Vec2(this.x * n, this.y * n);
    }

    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }

    add(v) {
        return new Vec2(this.x + v.x, this.y + v.y);
    }

    sub(v) {
        return new Vec2(this.x - v.x, this.y - v.y);
    }

    dot(v) {
        return new Vec2(this.x * v.x, this.y * v.y);
    }

    distance(target) {
        var v = this.sub(target);
        return v.length();
    }

    scale(factor) {
        return new Vec2(this.x * factor, this.y * factor);
    }

    lerp(target, alpha) {
        var a = this.scale(1 - alpha);
        var b = target.scale(alpha);
        return a.add(b);
    }
    toString() {
        return (
            this.x.toFixed(3).padStart(10) +
            this.y.toFixed(3).padStart(10));
    }
}
// *********
// Vec3
// *********

class Vec3 {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    normalize() {
        var n = this.length();
        if (n == 0) {
            return new Vec3(0, 0, 0);
        }
        n = 1.0 / n;
        return new Vec3(this.x * n, this.y * n, this.z * n);
    }

    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
    }

    add(v) {
        return new Vec3(this.x + v.x, this.y + v.y, this.z + v.z);
    }

    sub(v) {
        return new Vec3(this.x - v.x, this.y - v.y, this.z - v.z);
    }
    dot(v) {
        return new Vec3(this.x * v.x, this.y * v.y, this.z * v.z);
    }
    cross(v) {
        return new Vec3(
            this.y * v.z - this.z * v.y,
            -this.x * v.z + this.z * v.x,
            this.x * v.y - this.y * v.x
        );
    }

    distance(target) {
        var v = this.sub(target);
        return v.length();
    }

    scale(factor) {
        return new Vec3(this.x * factor, this.y * factor, this.z * factor);
    }

    lerp(target, alpha) {
        var a = this.scale(1 - alpha);
        var b = target.scale(alpha);
        return a.add(b);
    }
    toString() {
        return (
            this.x.toFixed(3).padStart(10) +
            this.y.toFixed(3).padStart(10) +
            this.z.toFixed(3).padStart(10));
    }
}

// *********
// Misc
// *********
if (!String.prototype.padStart) {
    String.prototype.padStart = function padStart(targetLength, padString) {
        targetLength = targetLength >> 0; //truncate if number, or convert non-number to 0;
        padString = String(typeof padString !== 'undefined' ? padString : ' ');
        if (this.length >= targetLength) {
            return String(this);
        } else {
            targetLength = targetLength - this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
            }
            return padString.slice(0, targetLength) + String(this);
        }
    };
}

if (!Number.prototype.clamp) {
    Number.prototype.clamp = function (min, max) {
        return Math.min(Math.max(this, min), max);
    };
}

